@@ -10,6 +10,34 @@
 #include <gnome-xml/tree.h>
 #include <gnome-xml/parser.h>
 
+int _gnome_print_debug = -1;
+
+#if 0
+#ifdef	__GNUC__
+#define debugmsg (format, args...) (_gnome_print_debug == -1 ? (getenv ("GNOME_PRINT_DEBUG") ? printf(format, ##args ), _gnome_print_debug = 1 : _gnome_print_debug = 0) : (_gnome_print_debug ? printf( format, ##args ) : 0))
+#else	/* !__GNUC__ */
+#endif
+#endif
+static void
+debugmsg (const gchar *format,
+	  ...)
+{
+  if ( _gnome_print_debug == -1 )
+    {
+      if ( getenv ("GNOME_PRINT_DEBUG") )
+	_gnome_print_debug = 1;
+      else
+	_gnome_print_debug = 0;
+    }
+  if ( _gnome_print_debug )
+    {
+      va_list args;
+      va_start (args, format);
+      vprintf (format, args);
+      va_end (args);
+    }
+}
+
 #define noVERBOSE
 
 typedef struct _GnomeFontType1EncTab GnomeFontType1EncTab;
@@ -21,11 +49,9 @@ struct _GnomeFontType1EncTab {
 
 static void gnome_font_class_init (GnomeFontClass *klass);
 
-static void
-gnome_font_init (GnomeFont *font);
+static void gnome_font_init (GnomeFont *font);
 
-static void
-gnome_font_finalize (GtkObject *object);
+static void gnome_font_finalize (GtkObject *object);
 
 static GtkObjectClass *parent_class = NULL;
 
@@ -223,16 +249,12 @@ gnome_font_is_afm_file (const char *fn)
   int len;
 
   len = strlen (fn);
-  return (len > 4 &&
-	  fn[len - 4] == '.' &&
-	  tolower (fn[len - 3]) == 'a' &&
-	  tolower (fn[len - 2]) == 'f' &&
-	  tolower (fn[len - 1]) == 'm');
+  return (len > 4 && strcasecmp (fn + len - 4, ".afm") == 0);
 }
 
 /* Try to find the filename for the font */
 static int
-gnome_font_find (GnomeFontClass *class, char *fontname)
+gnome_font_find (GnomeFontClass *class, const char *fontname)
 {
   int i;
 
@@ -243,7 +265,7 @@ gnome_font_find (GnomeFontClass *class, char *fontname)
 }
 
 static char *
-gnome_font_find_filename (GnomeFontClass *class, char *fontname)
+gnome_font_find_filename (GnomeFontClass *class, const char *fontname)
 {
   int i;
 
@@ -262,9 +284,10 @@ struct _GnomeFontWeightTab {
 };
 
 static void
-gnome_font_add_mapping (GnomeFontClass *class, char *fontname,
-			char *afm_fn, char *pfb_fn, char *fullname,
-			char *familyname, char *weight, char *alias)
+gnome_font_add_mapping (GnomeFontClass *class, const char *fontname,
+			const char *afm_fn, const char *pfb_fn,
+			const char *fullname, const char *familyname,
+			const char *weight, const char *alias)
 {
   int n_fonts;
   const GnomeFontWeightTab weight_tab[] = {
@@ -305,13 +328,13 @@ gnome_font_add_mapping (GnomeFontClass *class, char *fontname,
 
 
 #ifdef VERBOSE
-  g_print ("Adding %s %s\n", fontname, afm_fn);
+  debugmsg ("Adding %s %s\n", fontname, afm_fn);
 #endif
   if (gnome_font_find (class, fontname) >= 0)
     {
       /* already present in fontmap */
 #ifdef VERBOSE
-      g_print ("Font %s already present\n", fontname);
+      debugmsg ("Font %s already present\n", fontname);
 #endif
       return;
     }
@@ -322,7 +345,7 @@ gnome_font_add_mapping (GnomeFontClass *class, char *fontname,
   else if ((n_fonts & (n_fonts - 1)) == 0)
     {
 #ifdef VERBOSE
-      g_print ("reallocating to %d\n", n_fonts * 2);
+      debugmsg ("reallocating to %d\n", n_fonts * 2);
 #endif
       class->fontmap = g_realloc (class->fontmap,
 				  sizeof(GnomeFontMap) * n_fonts * 2);
@@ -371,7 +394,7 @@ gnome_font_refresh_font_dir (GnomeFontClass *class, char *dirname)
   FontInfo *fi;
   int status;
 
-  g_print (_("scanning dir %s\n"), dirname);
+  debugmsg (_("scanning dir %s\n"), dirname);
   dir = opendir (dirname);
   if (dir != NULL)
     {
@@ -382,14 +405,14 @@ gnome_font_refresh_font_dir (GnomeFontClass *class, char *dirname)
 	    break;
 	  if (gnome_font_is_afm_file (dir_ent->d_name))
 	    {
-	      g_print (_("checking file %s\n"), dir_ent->d_name);
+	      debugmsg (_("checking file %s\n"), dir_ent->d_name);
 	      fn = g_malloc (strlen (dirname) + strlen (dir_ent->d_name) + 2);
 	      sprintf (fn, _("%s/%s"), dirname, dir_ent->d_name);
 	      afm_f = fopen (fn, "rb");
 	      if (afm_f != NULL)
 		{
 		  status = parseFile (afm_f, &fi, P_G);
-		  g_print ("status = %d\n", status);
+		  debugmsg ("status = %d\n", status);
 		  if (status == 0)
 		    {
 		      gnome_font_add_mapping (class, fi->gfi->fontName, fn);
@@ -442,8 +465,12 @@ xmlGetValue (xmlNodePtr node, const char *name)
 	xmlNodePtr child;
 
 	ret = (char *) xmlGetProp (node, name);
-	if (ret != NULL)
-		return g_strdup (ret);
+	if (ret != NULL) {
+		char *gret;
+		gret = g_strdup (ret);
+		free (ret);
+		return gret;
+	}
 	child = node->childs;
 
 	while (child != NULL) {
@@ -468,16 +495,12 @@ xmlGetValue (xmlNodePtr node, const char *name)
 static void
 gnome_font_load_fontmap (GnomeFontClass *class, const char *fn)
 {
-  FILE *f;
-  char linebuf[1024];
-  char *p;
   char *type;
   char *fontname, *afmfile, *pfbfile, *fullname, *familyname, *weight, *alias;
-  char *subst_font_name;
   xmlDoc *doc;
 
 #ifdef VERBOSE
-  g_print ("filename %s\n", fn);
+  debugmsg ("filename %s\n", fn);
 #endif
 
   doc = xmlParseFile( fn );
@@ -527,9 +550,10 @@ gnome_font_load_fontmap (GnomeFontClass *class, const char *fn)
                             weight ? weight : "**missing**");
                 }
 #ifdef VERBOSE
-	      printf ("%s %s %s %s %s %s\n",
-		      fontname, afmfile, pfbfile, fullname, familyname, weight);
+	      debugmsg ("%s %s %s %s %s %s\n",
+			fontname, afmfile, pfbfile, fullname, familyname, weight);
 #endif
+	      g_free (alias);
 	      g_free (weight);
 	      g_free (familyname);
 	      g_free (fullname);
@@ -541,6 +565,8 @@ gnome_font_load_fontmap (GnomeFontClass *class, const char *fn)
 	  font = font->next;
 	}
     }
+  if (doc)
+    xmlFreeDoc (doc);
 }
 
 static void
@@ -690,7 +716,7 @@ gnome_font_load_afm (GnomeFontMap *fontmap_entry)
     {
       status = parseFile (afm_f, &fi, P_M | P_P);
 #ifdef VERBOSE
-      g_print (_("status loading %s = %d\n"), fontmap_entry->afm_fn, status);
+      debugmsg (_("status loading %s = %d\n"), fontmap_entry->afm_fn, status);
 #endif
       if (status == 0)
 	{
@@ -785,7 +811,7 @@ gnome_font_load_afm (GnomeFontMap *fontmap_entry)
 	      ktab[j].glyph2 = glyph2;
 	      ktab[j].x_amt = fi->pkd[i].xamt;
 #ifdef VERBOSE
-	      g_print ("kern pair %s(%d) %s(%d) %d\n",
+	      debugmsg ("kern pair %s(%d) %s(%d) %d\n",
 		       fi->pkd[i].name1, glyph1,
 		       fi->pkd[i].name2, glyph2,
 		       fi->pkd[i].xamt);
@@ -794,11 +820,16 @@ gnome_font_load_afm (GnomeFontMap *fontmap_entry)
 
 #ifdef VERBOSE
 	  for (i = 0; i < ktabsize; i++)
-	    g_print ("%d %d %d\n",
+	    debugmsg ("%d %d %d\n",
 		     ktab[i].glyph1, ktab[i].glyph2, ktab[i].x_amt);
 #endif
 
 	  /* Free the Font_Info structure */
+	  for (i = 0; i < fi->numOfPairs; i++)
+	    {
+	      free (fi->pkd[i].name1);
+	      free (fi->pkd[i].name2);
+	    }
 	  free (fi->pkd);
 	  for (i = 0; i < fi->numOfChars; i++)
 	    {
@@ -1015,8 +1046,6 @@ gnome_font_unsized_get_name (GnomeFontUnsized *font)
 char *
 gnome_font_unsized_get_glyph_name (GnomeFontUnsized *font)
 {
-  GnomeFontMap *subst;
-
   g_return_val_if_fail (font != NULL, NULL);
 
   if (font->alias)
@@ -1138,7 +1167,9 @@ gnome_font_unsized_get_pfa (GnomeFontUnsized *font)
 {
   char *pfb_fn;
   FILE *f;
+#if 0
   GnomeFontMap *subst;
+#endif
 
   char *pfb;
   int pfb_size, pfb_size_max;
@@ -1235,6 +1266,20 @@ gnome_font_get_width_string (GnomeFont *font, const char *s)
   return width * font->scale;
 }
 
+/* This works with the standard Adobe encoding and without kerning or
+   ligatures. When the text libs get written, this function will be
+   deprecated. */
+double
+gnome_font_get_width_string_n (GnomeFont *font, const char *s, int n)
+{
+  unsigned char *us = (unsigned char *)s;
+  int const * const widths = font->fontmap_entry->widths;
+  double width = 0.0;
+  while (--n >= 0)
+      width += widths[*(us++) ];
+  return width * font->scale;
+}
+
 /* Get the glyph number corresponding to a given unicode, or -1 if it
    is not mapped. */
 int
@@ -1369,47 +1414,47 @@ gnome_font_family_list_free (GList *fontlist)
 #define GNOME_FONT_EXTRA_BLACK GNOME_FONT_EXTRABLACK
 #endif
 
-static char *getFontComponent (char *font_name, unsigned int pos);
-static char *setFontComponent (char *font_name, unsigned int pos, char *s);
+static char *getFontComponent (const char *font_name, unsigned int pos);
+static char *setFontComponent (const char *font_name, unsigned int pos, const char *s);
 
-static char *getFontFoundry (char *font_name);
-static char *getFontFamily (char *font_name);
-static char *getFontWeight (char *font_name);
-static GnomeFontWeight getGnomeFontWeight (char *font_name);
-static char *getFontSlant (char *font_name);
-static gboolean fontIsItalic (char *font_name);
-static char *getFontSWidth (char *font_name);
-static char *getFontAdStyle (char *font_name);
-static int getFontPixelSize (char *font_name);
-static int getFontPointSize (char *font_name);
-static int getFontResolutionX (char *font_name);
-static int getFontResolutionY (char *font_name);
-static char *getFontSpace (char *font_name);
-static int getFontAverageWidth (char *font_name);
-static char *getFontRegistry (char *font_name);
-static char *getFontEncoding (char *font_name);
+static char *getFontFoundry (const char *font_name);
+static char *getFontFamily (const char *font_name);
+static char *getFontWeight (const char *font_name);
+static GnomeFontWeight getGnomeFontWeight (const char *font_name);
+static char *getFontSlant (const char *font_name);
+static gboolean fontIsItalic (const char *font_name);
+static char *getFontSWidth (const char *font_name);
+static char *getFontAdStyle (const char *font_name);
+static int getFontPixelSize (const char *font_name);
+static int getFontPointSize (const char *font_name);
+static int getFontResolutionX (const char *font_name);
+static int getFontResolutionY (const char *font_name);
+static char *getFontSpace (const char *font_name);
+static int getFontAverageWidth (const char *font_name);
+static char *getFontRegistry (const char *font_name);
+static char *getFontEncoding (const char *font_name);
 
 
 /* these malloc, so free what they return */
 
-static char *setFontFoundry (char *font_name, char *s);
-static char *setFontFamily (char *font_name, char *s);
-static char *setFontWeight (char *font_name, char *s);
-static char *setFontSlant (char *font_name, char *s);
-static char *setFontSWidth (char *font_name, char *s);
-static char *setFontAdStyle (char *font_name, char *s);
-static char *setFontPixelSize (char *font_name, int i);
-static char *setFontPointSize (char *font_name, int i);
-static char *setFontResolutionX (char *font_name, int i);
-static char *setFontResolutionY (char *font_name, int i);
-static char *setFontSpace (char *font_name, char *s);
-static char *setFontAverageWidth (char *font_name, int i);
-static char *setFontRegistry (char *font_name, char *s);
-static char *setFontEncoding (char *font_name, char *s);
+static char *setFontFoundry (const char *font_name, const char *s);
+static char *setFontFamily (const char *font_name, const char *s);
+static char *setFontWeight (const char *font_name, const char *s);
+static char *setFontSlant (const char *font_name, const char *s);
+static char *setFontSWidth (const char *font_name, const char *s);
+static char *setFontAdStyle (const char *font_name, const char *s);
+static char *setFontPixelSize (const char *font_name, int i);
+static char *setFontPointSize (const char *font_name, int i);
+static char *setFontResolutionX (const char *font_name, int i);
+static char *setFontResolutionY (const char *font_name, int i);
+static char *setFontSpace (const char *font_name, const char *s);
+static char *setFontAverageWidth (const char *font_name, int i);
+static char *setFontRegistry (const char *font_name, const char *s);
+static char *setFontEncoding (const char *font_name, const char *s);
 
 static void setComponentReplace (char **io,
-				 char *(*func) (char *font_name, char *s),
-				 char *part);
+				 char *(*func) (const char *font_name, const char *s),
+				 const char *part);
 
 
 GHashTable *scaled_display_fonts = NULL;
@@ -1478,7 +1523,8 @@ gnome_font_weight_to_string_map gnome_font_weight_to_string_maps [] =
 };
 
 
-static void initialize_hashes ()
+static void
+initialize_hashes (void)
 {
   gnome_font_family_to_x_map *ftx = gnome_font_family_to_x_maps;
   gnome_font_weight_to_string_map *wtx = gnome_font_weight_to_string_maps;
@@ -1494,8 +1540,8 @@ static void initialize_hashes ()
   /* build 2 way hashes for x vs font family names */
   while (ftx->print_name)
     {
-      printf ("inserting family-->x mapping: '%s' --> '%s'\n",
-	      ftx->print_name, ftx->x_name);
+      debugmsg ("inserting family-->x mapping: '%s' --> '%s'\n",
+		ftx->print_name, ftx->x_name);
       g_hash_table_insert (gnome_font_family_to_x_hash,
 			   ftx->print_name, ftx->x_name);
       g_hash_table_insert (gnome_font_x_to_family_hash,
@@ -1507,7 +1553,7 @@ static void initialize_hashes ()
   /* build 2 way hashes for gnome font weight to x names */
   while (wtx->name)
     {
-      printf ("inserting weight-->string mapping: '%d' --> '%s'\n",
+      debugmsg ("inserting weight-->string mapping: '%d' --> '%s'\n",
 	      wtx->weight, wtx->name);
       g_hash_table_insert (gnome_font_weight_to_string_hash,
 			   & (wtx->weight), wtx->name);
@@ -1518,12 +1564,13 @@ static void initialize_hashes ()
 }
 
 
-char *gnome_font_family_to_x_name (char *family)
+const char *
+gnome_font_family_to_x_name (const char *family)
 {
-  char *x_name;
+  const char *x_name;
   if (! scaled_display_fonts) initialize_hashes ();
 
-  x_name = (char *) g_hash_table_lookup (gnome_font_family_to_x_hash, family);
+  x_name = (const char *) g_hash_table_lookup (gnome_font_family_to_x_hash, family);
   if (! x_name)
     return "helvetica";
 
@@ -1531,12 +1578,13 @@ char *gnome_font_family_to_x_name (char *family)
 }
 
 
-char *gnome_font_x_name_to_family (char *x_name)
+const char *
+gnome_font_x_name_to_family (const char *x_name)
 {
-  char *family;
+  const char *family;
   if (! scaled_display_fonts) initialize_hashes ();
 
-  family = (char *) g_hash_table_lookup (gnome_font_x_to_family_hash, x_name);
+  family = (const char *) g_hash_table_lookup (gnome_font_x_to_family_hash, x_name);
   if (! family)
     return "Helvetica";
 
@@ -1544,13 +1592,14 @@ char *gnome_font_x_name_to_family (char *x_name)
 }
 
 
-char *gnome_font_weight_to_string (GnomeFontWeight gfw)
+const char *
+gnome_font_weight_to_string (GnomeFontWeight gfw)
 {
-  char *x_weight;
+  const char *x_weight;
   if (! scaled_display_fonts) initialize_hashes ();
 
   x_weight =
-    (char *) g_hash_table_lookup (gnome_font_weight_to_string_hash, &gfw);
+    (const char *) g_hash_table_lookup (gnome_font_weight_to_string_hash, &gfw);
 
   if (! x_weight)
     x_weight = "medium";
@@ -1559,7 +1608,8 @@ char *gnome_font_weight_to_string (GnomeFontWeight gfw)
 }
 
 
-GnomeFontWeight string_to_gnome_font_weight (char *x_weight)
+GnomeFontWeight
+string_to_gnome_font_weight (const char *x_weight)
 {
   GnomeFontWeight *gfw;
 
@@ -1580,12 +1630,12 @@ static int iabs (int a)
 }
 
 static GdkFont *
-find_best_x_weight (char *x_font_name_ret, GnomeFontWeight start_weight)
+find_best_x_weight (const char *x_font_name_ret, GnomeFontWeight start_weight)
 {
   gchar *current_x_name = g_strdup( x_font_name_ret );
   GdkFont *current_font = gdk_font_load (current_x_name);
   GnomeFontWeight weighta, weightb;
-  char *x_weight;
+  const char *x_weight;
 
   if (! scaled_display_fonts) initialize_hashes ();
 
@@ -1601,7 +1651,7 @@ find_best_x_weight (char *x_font_name_ret, GnomeFontWeight start_weight)
       weightb ++;
 
       x_weight =
-	(char *) g_hash_table_lookup (gnome_font_weight_to_string_hash, &weighta);
+	(const char *) g_hash_table_lookup (gnome_font_weight_to_string_hash, &weighta);
 
       if ( x_weight )
 	{
@@ -1615,7 +1665,7 @@ find_best_x_weight (char *x_font_name_ret, GnomeFontWeight start_weight)
 	}
 
       x_weight =
-	(char *) g_hash_table_lookup (gnome_font_weight_to_string_hash, &weightb);
+	(const char *) g_hash_table_lookup (gnome_font_weight_to_string_hash, &weightb);
 
       if ( x_weight )
 	{
@@ -1637,14 +1687,14 @@ find_best_x_weight (char *x_font_name_ret, GnomeFontWeight start_weight)
 
 /* here is a horrible little routine that tries to find an X font
    that is about the same size as a printer font */
-static GdkFont *find_best_x_font (char *unsized_name,
+static GdkFont *find_best_x_font (const char *unsized_name,
 				  GnomeFont *gnome_font,
 				  char **x_font_name_ret)
 {
   /* sample string used during the search for a good size */
   /*char *test_string = "abcdefghijklmnopqrstuvwxyz";*/
   /*char *test_string = "snider";*/
-  char *test_string = "oooowwww";
+  const char *test_string = "oooowwww";
   int length = strlen (test_string);
 
   double perfect_width; /* the printer-font width of the sample string */
@@ -1668,7 +1718,7 @@ static GdkFont *find_best_x_font (char *unsized_name,
   /*space_width = gnome_font_get_width_string (gnome_font, " ");*/
   /*max_width = perfect_width + space_width / 2;*/
   max_width = perfect_width /*- 1*/;
-  g_print( "perfect_width = %f\n", perfect_width );
+  debugmsg( "perfect_width = %f\n", perfect_width );
 #endif
   current_size = (int) (gnome_font->size);
   current_x_name = setFontPixelSize (unsized_name, current_size);
@@ -1677,19 +1727,19 @@ static GdkFont *find_best_x_font (char *unsized_name,
   /* start by getting some x font loaded */
   if (current_font == NULL)
     {
-      printf ("couldn't load first font '%s'\n", current_x_name);
+      debugmsg ("couldn't load first font '%s'\n", current_x_name);
       /* is it italic? */
       if (gnome_font->fontmap_entry->italic)
 	{
 	  /* try oblique */
-	  printf ("trying oblique...\n");
+	  debugmsg ("trying oblique...\n");
 	  setComponentReplace (&current_x_name, setFontSlant, "o");
 	  current_font = find_best_x_weight (current_x_name, gnome_font->fontmap_entry->weight_code);
 
 	  if (current_font == NULL)
 	    {
 	      /* failed.  put italic back */
-	      printf ("oblique failed: '%s'\n", current_x_name);
+	      debugmsg ("oblique failed: '%s'\n", current_x_name);
 	      setComponentReplace (&current_x_name, setFontSlant, "i");
 	    }
 	}
@@ -1697,25 +1747,25 @@ static GdkFont *find_best_x_font (char *unsized_name,
       if (current_font == NULL)
 	{
 	  /* try helvetica */
-	  printf ("trying hevletica...\n");
+	  debugmsg ("trying hevletica...\n");
 	  setComponentReplace (&current_x_name, setFontFamily, "helvetica");
 	  current_font = find_best_x_weight (current_x_name, gnome_font->fontmap_entry->weight_code);
 
 	  if (current_font == NULL)
 	    {
-	      printf ("helvetica failed. giving up...\n");
+	      debugmsg ("helvetica failed. giving up...\n");
 	      return NULL;
 	    }
 	}
     }
 
-  printf ("initial name loaded: '%s'\n", current_x_name);
+  debugmsg ("initial name loaded: '%s'\n", current_x_name);
 
 #if 0
   current_width = gdk_text_width (current_font, test_string, length);
   current_diff = (int) (perfect_width - current_width);
 
-  printf ("size=%d, starting diff=%d\n", current_size, current_diff);
+  debugmsg ("size=%d, starting diff=%d\n", current_size, current_diff);
 
   /* are we looking for a bigger x font or a smaller one? */
   /*
@@ -1727,14 +1777,14 @@ static GdkFont *find_best_x_font (char *unsized_name,
       /*direction = (int) gnome_font->size / 6.0;*/
       direction = 10;
       /*if (direction < 1) direction = 1;*/
-      printf ("size=%f, direction=%d\n", gnome_font->size, direction);
+      debugmsg ("size=%f, direction=%d\n", gnome_font->size, direction);
     }
   else
     {
       /*direction = (int) gnome_font->size / -6.0;*/
       direction = -10;
       /*if (direction > -1) direction = -1;*/
-      printf ("size=%f, direction=%d\n", gnome_font->size, direction);
+      debugmsg ("size=%f, direction=%d\n", gnome_font->size, direction);
     }
 
 
@@ -1748,7 +1798,7 @@ static GdkFont *find_best_x_font (char *unsized_name,
 
       if (next_font == NULL)
 	{
-	  printf ("couldn't load '%s'\n", next_x_name);
+	  debugmsg ("couldn't load '%s'\n", next_x_name);
 	  continue; /* ??? */
 	}
       next_width = gdk_text_width (next_font, test_string, length);
@@ -1757,7 +1807,7 @@ static GdkFont *find_best_x_font (char *unsized_name,
       /* if we've gotten bigger than max_width, stop */
       if (next_width >= max_width && direction > 0)
 	{
-	  printf (" bailing because next_width[%d] >= max_width[%d]\n",
+	  debugmsg (" bailing because next_width[%d] >= max_width[%d]\n",
 		  next_width, (int) max_width);
 	  break;
 	}
@@ -1765,7 +1815,7 @@ static GdkFont *find_best_x_font (char *unsized_name,
       /* if the previous font was closer than this one, stop */
       if (iabs (next_diff) > iabs (current_diff))
 	{
-	  printf (" bailing because next_diff[%d] > current_diff[%d]\n",
+	  debugmsg (" bailing because next_diff[%d] > current_diff[%d]\n",
 		  next_diff, current_diff);
 	  break;
 	}
@@ -1778,7 +1828,7 @@ static GdkFont *find_best_x_font (char *unsized_name,
       current_font = next_font;
       current_diff = next_diff;
 
-      printf (" new size=%d, new diff=%d\n", current_size, current_diff);
+      debugmsg (" new size=%d, new diff=%d\n", current_size, current_diff);
     }
 #endif
   (*x_font_name_ret) = current_x_name;
@@ -1789,14 +1839,14 @@ static GdkFont *find_best_x_font (char *unsized_name,
 
 /* construct an x font name and find the best point size */
 
-static GnomeDisplayFont *create_display_font (char *family,
+static GnomeDisplayFont *create_display_font (const char *family,
 					      GnomeFontWeight weight,
 					      gboolean italic,
 					      double points,
 					      double scale)
 {
-  char *x_family;
-  char *xfn = g_strdup ("-*-helvetica-*-r-*-*-12-*-*-*-*-*-*-*");
+  const char *x_family;
+  char *xfn;
   GnomeFontUnsized *gfus;
   GnomeFont *gnome_font, *closest;
   GnomeDisplayFont *wpdf;
@@ -1807,7 +1857,7 @@ static GnomeDisplayFont *create_display_font (char *family,
 
   if ((closest = gnome_font_new_closest (family, weight, italic, 0)) == NULL)
 	  return NULL;
-	
+
   gfus = closest->fontmap_entry;
   gnome_font = gnome_font_new_closest (family, weight, italic, points * scale);
   wpdf = (GnomeDisplayFont *) g_malloc (sizeof (GnomeDisplayFont));
@@ -1817,6 +1867,7 @@ static GnomeDisplayFont *create_display_font (char *family,
   wpdf->scale = scale;
 
   x_family = gnome_font_family_to_x_name (gfus->familyname);
+  xfn = g_strdup ("-*-helvetica-*-r-*-*-12-*-*-*-*-*-*-*");
   setComponentReplace (&xfn, setFontFamily, x_family);
   setComponentReplace (&xfn, setFontWeight,
 		       gnome_font_weight_to_string (gfus->weight_code));
@@ -1835,7 +1886,7 @@ static GnomeDisplayFont *create_display_font (char *family,
 
 /* cache for create_display_font */
 
-GnomeDisplayFont *gnome_get_display_font (char *family,
+GnomeDisplayFont *gnome_get_display_font (const char *family,
 					  GnomeFontWeight weight,
 					  gboolean italic,
 					  double points,
@@ -1853,7 +1904,7 @@ GnomeDisplayFont *gnome_get_display_font (char *family,
   if (! scaled_display_fonts) initialize_hashes ();
 
   /*
-  printf ("lookup_display_font -- hash:%p key='%s'\n",
+  debugmsg ("lookup_display_font -- hash:%p key='%s'\n",
 	  scaled_display_fonts,
 	  key);
   */
@@ -1863,8 +1914,10 @@ GnomeDisplayFont *gnome_get_display_font (char *family,
   if (wpdf == NULL)
     {
       wpdf = create_display_font (family, weight, italic, points, scale);
-      g_return_val_if_fail (wpdf, NULL);
-      g_hash_table_insert (scaled_display_fonts, g_strdup (key), wpdf);
+      if (wpdf)
+	g_hash_table_insert (scaled_display_fonts, g_strdup (key), wpdf);
+      else
+	/* We should maintain a negative cache.  */;
     }
 
   return wpdf;
@@ -1903,7 +1956,7 @@ GnomeDisplayFont *gnome_font_get_display_font( GnomeFont *font )
 */
 
 
-char *getFontComponent (char *font_name, unsigned int pos)
+char *getFontComponent (const char *font_name, unsigned int pos)
 {
   char *p, *parts[ 14 ];
   char *scratch;
@@ -1938,9 +1991,11 @@ char *getFontComponent (char *font_name, unsigned int pos)
 }
 
 
-char *setFontComponent (char *font_name, unsigned int pos, char *s)
+char *
+setFontComponent (const char *font_name, unsigned int pos, const char *s)
 {
-  char *p, *parts[ 14 ];
+  char *p;
+  const char *parts[ 14 ];
   char *scratch;
   char *new_name;
   int i, len;
@@ -1990,22 +2045,26 @@ char *setFontComponent (char *font_name, unsigned int pos, char *s)
 }
 
 
-char *getFontFoundry (char *font_name)
+static char *
+getFontFoundry (const char *font_name)
 {
   return getFontComponent (font_name, 0);
 }
 
-char *getFontFamily (char *font_name)
+static char *
+getFontFamily (const char *font_name)
 {
   return getFontComponent (font_name, 1);
 }
 
-char *getFontWeight (char *font_name)
+static char *
+getFontWeight (const char *font_name)
 {
   return getFontComponent (font_name, 2);
 }
 
-GnomeFontWeight getGnomeFontWeight (char *font_name)
+static GnomeFontWeight
+getGnomeFontWeight (const char *font_name)
 {
   char *weight = getFontComponent (font_name, 2);
   GnomeFontWeight fw = string_to_gnome_font_weight (weight);
@@ -2013,12 +2072,14 @@ GnomeFontWeight getGnomeFontWeight (char *font_name)
   return fw;
 }
 
-char *getFontSlant (char *font_name)
+static char *
+getFontSlant (const char *font_name)
 {
   return getFontComponent (font_name, 3);
 }
 
-gboolean fontIsItalic (char *font_name)
+static gboolean
+fontIsItalic (const char *font_name)
 {
   char *slant = getFontComponent (font_name, 3);
   gboolean s = FALSE;
@@ -2031,17 +2092,20 @@ gboolean fontIsItalic (char *font_name)
   return s;
 }
 
-char *getFontSWidth (char *font_name)
+static char *
+getFontSWidth (const char *font_name)
 {
   return getFontComponent (font_name, 4);
 }
 
-char *getFontAdStyle (char *font_name)
+static char *
+getFontAdStyle (const char *font_name)
 {
   return getFontComponent (font_name, 5);
 }
 
-int getFontPixelSize (char *font_name)
+static int
+getFontPixelSize (const char *font_name)
 {
   int ret;
   char *p = getFontComponent (font_name, 6);
@@ -2050,7 +2114,8 @@ int getFontPixelSize (char *font_name)
   return ret;
 }
 
-int getFontPointSize (char *font_name)
+static int
+getFontPointSize (const char *font_name)
 {
   int ret;
   char *p = getFontComponent (font_name, 7);
@@ -2059,7 +2124,8 @@ int getFontPointSize (char *font_name)
   return ret;
 }
 
-int getFontResolutionX (char *font_name)
+static int
+getFontResolutionX (const char *font_name)
 {
   int ret;
   char *p = getFontComponent (font_name, 8);
@@ -2068,7 +2134,8 @@ int getFontResolutionX (char *font_name)
   return ret;
 }
 
-int getFontResolutionY (char *font_name)
+static int
+getFontResolutionY (const char *font_name)
 {
   int ret;
   char *p = getFontComponent (font_name, 9);
@@ -2077,12 +2144,14 @@ int getFontResolutionY (char *font_name)
   return ret;
 }
 
-char *getFontSpace (char *font_name)
+static char *
+getFontSpace (const char *font_name)
 {
   return getFontComponent (font_name, 10);
 }
 
-int getFontAverageWidth (char *font_name)
+static int
+getFontAverageWidth (const char *font_name)
 {
   int ret;
   char *p = getFontComponent (font_name, 11);
@@ -2091,12 +2160,14 @@ int getFontAverageWidth (char *font_name)
   return ret;
 }
 
-char *getFontRegistry (char *font_name)
+static char *
+getFontRegistry (const char *font_name)
 {
   return getFontComponent (font_name, 12);
 }
 
-char *getFontEncoding (char *font_name)
+static char *
+getFontEncoding (const char *font_name)
 {
   return getFontComponent (font_name, 13);
 }
@@ -2104,104 +2175,114 @@ char *getFontEncoding (char *font_name)
 
 
 
-char *setFontFoundry (char *font_name, char *s)
+static char *
+setFontFoundry (const char *font_name, const char *s)
 {
   return setFontComponent (font_name, 0, s);
 }
 
-char *setFontFamily (char *font_name, char *s)
+static char *
+setFontFamily (const char *font_name, const char *s)
 {
   return setFontComponent (font_name, 1, s);
 }
 
-char *setFontWeight (char *font_name, char *s)
+static char *
+setFontWeight (const char *font_name, const char *s)
 {
   return setFontComponent (font_name, 2, s);
 }
 
-char *setFontSlant (char *font_name, char *s)
+static char *
+setFontSlant (const char *font_name, const char *s)
 {
   return setFontComponent (font_name, 3, s);
 }
 
-char *setFontSWidth (char *font_name, char *s)
+static char *
+setFontSWidth (const char *font_name, const char *s)
 {
   return setFontComponent (font_name, 4, s);
 }
 
-char *setFontAdStyle (char *font_name, char *s)
+static char *
+setFontAdStyle (const char *font_name, const char *s)
 {
   return setFontComponent (font_name, 5, s);
 }
 
-char *setFontPixelSize (char *font_name, int i)
+static char *
+setFontPixelSize (const char *font_name, int i)
 {
-  gchar *temp;
+  gchar temp[4 * sizeof (int)];
   gchar *returnval;
-  temp = g_strdup_printf( "%d", i );
+  sprintf (temp, "%d", i );
   returnval = setFontComponent (font_name, 6, temp);
-  g_free( temp );
   return returnval;
 }
 
-char *setFontPointSize (char *font_name, int i)
+static char *
+setFontPointSize (const char *font_name, int i)
 {
-  gchar *temp;
+  gchar temp[4 * sizeof (int)];
   gchar *returnval;
-  temp = g_strdup_printf( "%d", i );
+  sprintf (temp, "%d", i );
   returnval = setFontComponent (font_name, 7, temp);
-  g_free( temp );
   return returnval;
 }
 
-char *setFontResolutionX (char *font_name, int i)
+static char *
+setFontResolutionX (const char *font_name, int i)
 {
-  gchar *temp;
+  gchar temp[4 * sizeof (int)];
   gchar *returnval;
-  temp = g_strdup_printf( "%d", i );
+  sprintf (temp, "%d", i );
   returnval = setFontComponent (font_name, 8, temp);
-  g_free( temp );
   return returnval;
 }
 
-char *setFontResolutionY (char *font_name, int i)
+static char *
+setFontResolutionY (const char *font_name, int i)
 {
-  gchar *temp;
+  gchar temp[4 * sizeof (int)];
   gchar *returnval;
-  temp = g_strdup_printf( "%d", i );
+  sprintf (temp, "%d", i );
   returnval = setFontComponent (font_name, 9, temp);
-  g_free( temp );
   return returnval;
 }
 
-char *setFontSpace (char *font_name, char *s)
+static char *
+setFontSpace (const char *font_name, const char *s)
 {
   return setFontComponent (font_name, 10, s);
 }
 
-char *setFontAverageWidth (char *font_name, int i)
+static char *
+setFontAverageWidth (const char *font_name, int i)
 {
-  gchar *temp;
+  gchar temp[4 * sizeof (int)];
   gchar *returnval;
-  temp = g_strdup_printf( "%d", i );
+  sprintf (temp, "%d", i );
   returnval = setFontComponent (font_name, 11, temp);
-  g_free( temp );
   return returnval;
 }
 
-char *setFontRegistry (char *font_name, char *s)
+static char *
+setFontRegistry (const char *font_name, const char *s)
 {
   return setFontComponent (font_name, 12, s);
 }
 
-char *setFontEncoding (char *font_name, char *s)
+static char *
+setFontEncoding (const char *font_name, const char *s)
 {
   return setFontComponent (font_name, 13, s);
 }
 
-static void setComponentReplace (char **io,
-				 char *(*func) (char *font_name, char *s),
-				 char *part)
+static void
+setComponentReplace (char **io,
+		     char *(*func) (const char *font_name, const char *s),
+		     const char *part)
 {
   char *new_name = (*func) (*io, part);
   g_free (*io);
